@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef PAUSEMUSICPLUGIN_H
@@ -39,12 +39,12 @@ class PauseMusicPlugin
 public:
     explicit PauseMusicPlugin(QObject* parent, const QVariantList& args);
 
-    bool receivePackage(const NetworkPackage& np) override;
+    bool receivePacket(const NetworkPacket& np) override;
     void connected() override { }
     
 private:
     QSet<QString> pausedSources;
-    bool muted;
+    QSet<QString> mutedSinks;
 
 };
 
