@@ -15,14 +15,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef SHARE_CONFIG_H
-#define SHARE_CONFIG_H
+#ifndef RUNCOMMAND_CONFIG_H
+#define RUNCOMMAND_CONFIG_H
 
 #include "kcmplugin/kdeconnectpluginkcm.h"
 
+class QMenu;
 class QStandardItemModel;
 
 class RunCommandConfig
@@ -41,10 +42,12 @@ public Q_SLOTS:
 private Q_SLOTS:
     void onDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
 private:
+    void addSuggestedCommand(QMenu* menu, const QString &name, const QString &command);
+    void insertRow(int i, const QString &name, const QString &command);
     void insertEmptyRow();
 
     QStandardItemModel* m_entriesModel;
 
 };
 
-#endif
+#endif // RUNCOMMAND_CONFIG_H
