@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "../core/backends/lan/server.h"
@@ -25,6 +25,7 @@
 #include <QtCrypto>
 #include <QTest>
 #include <QTimer>
+#include <QSignalSpy>
 
 /*
  * This class tests the behaviour of socket line reader when the connection if over ssl. Since SSL takes part below application layer,
@@ -34,7 +35,8 @@ class TestSslSocketLineReader : public QObject
 {
     Q_OBJECT
 public Q_SLOTS:
-    void newPackage();
+    void newPacket();
+    void testTimeout();
 
 private Q_SLOTS:
 
@@ -50,10 +52,11 @@ private Q_SLOTS:
 
 private:
     const int PORT = 7894;
+    const int TIMEOUT = 4 * 1000;
     QTimer m_timer;
     QCA::Initializer m_qcaInitializer;
     QEventLoop m_loop;
-    QList<QByteArray> m_packages;
+    QList<QByteArray> m_packets;
     Server* m_server;
     QSslSocket* m_clientSocket;
     SocketLineReader* m_reader;
@@ -68,20 +71,20 @@ void TestSslSocketLineReader::initTestCase()
 
     QVERIFY2(m_server->listen(QHostAddress::LocalHost, PORT), "Failed to create local tcp server");
 
-    m_timer.setInterval(10 * 1000);//Ten second is more enough for this test, just used this so that to break mLoop if stuck
     m_timer.setSingleShot(true);
-    connect(&m_timer, &QTimer::timeout, &m_loop, &QEventLoop::quit);
-
-    m_timer.start();
+    connect(&m_timer, &QTimer::timeout, this, &TestSslSocketLineReader::testTimeout);
 }
 
 void TestSslSocketLineReader::init()
 {
+    m_timer.setInterval(TIMEOUT);
+    m_timer.start();
+
     m_clientSocket = new QSslSocket(this);
     m_clientSocket->connectToHost(QHostAddress::LocalHost, PORT);
     connect(m_clientSocket, &QAbstractSocket::connected, &m_loop, &QEventLoop::quit);
 
-    m_loop.exec();
+    m_loop.processEvents(QEventLoop::AllEvents, TIMEOUT);
 
     QVERIFY2(m_clientSocket->isOpen(), "Could not connect to local tcp server");
 }
@@ -89,6 +92,7 @@ void TestSslSocketLineReader::init()
 void TestSslSocketLineReader::cleanup()
 {
     m_clientSocket->disconnectFromHost();
+    m_timer.stop();
     delete m_clientSocket;
 }
 
@@ -101,14 +105,16 @@ void TestSslSocketLineReader::testTrustedDevice()
 {
 
     int maxAttemps = 5;
-    QCOMPARE(true, m_server->hasPendingConnections());
     while(!m_server->hasPendingConnections() && maxAttemps > 0) {
         --maxAttemps;
         QTest::qSleep(1000);
     }
+    QCOMPARE(true, m_server->hasPendingConnections());
 
     QSslSocket* serverSocket = m_server->nextPendingConnection();
 
+    QSignalSpy serverEncryptedSpy(serverSocket, SIGNAL(encrypted()));
+
     QVERIFY2(serverSocket != 0, "Null socket returned by server");
     QVERIFY2(serverSocket->isOpen(), "Server socket already closed");
 
@@ -123,10 +129,28 @@ void TestSslSocketLineReader::testTrustedDevice()
     m_clientSocket->setPeerVerifyMode(QSslSocket::VerifyPeer);
     m_clientSocket->addCaCertificate(serverSocket->localCertificate());
 
-    connect(m_clientSocket, &QSslSocket::encrypted, &m_loop, &QEventLoop::quit);
+    int connected_sockets = 0;
+    auto connected_lambda = [&](){
+        connected_sockets++;
+        if (connected_sockets >= 2) {
+            m_loop.quit();
+        }
+    };
+    connect(serverSocket, &QSslSocket::encrypted, connected_lambda);
+    connect(m_clientSocket, &QSslSocket::encrypted, connected_lambda);
+
     serverSocket->startServerEncryption();
     m_clientSocket->startClientEncryption();
-    m_loop.exec();
+    m_loop.exec(); //Block until QEventLoop::quit gets called by the lambda
+
+    if (serverEncryptedSpy.count() < 1) {
+        for(int x = 0;x < 50; ++x) {
+            QVERIFY(serverEncryptedSpy.wait(100));
+            if (serverEncryptedSpy.count() > 0) {
+                break;
+            }
+        }
+    }
 
     // Both client and server socket should be encrypted here and should have remote certificate because VerifyPeer is used
     QVERIFY2(m_clientSocket->isOpen(), "Client socket already closed");
@@ -138,23 +162,23 @@ void TestSslSocketLineReader::testTrustedDevice()
 
     QList<QByteArray> dataToSend;
     dataToSend << "foobar\n" << "barfoo\n" << "foobar?\n" << "\n" << "barfoo!\n" << "panda\n";
-    for (const QByteArray& line : dataToSend) {
+    for (const QByteArray& line : qAsConst(dataToSend)) {
         m_clientSocket->write(line);
     }
     m_clientSocket->flush();
 
-    m_packages.clear();
+    m_packets.clear();
 
     m_reader = new SocketLineReader(serverSocket, this);
-    connect(m_reader, &SocketLineReader::readyRead, this,&TestSslSocketLineReader::newPackage);
+    connect(m_reader, &SocketLineReader::readyRead, this,&TestSslSocketLineReader::newPacket);
     m_loop.exec();
 
     /* remove the empty line before compare */
     dataToSend.removeOne("\n");
 
-    QCOMPARE(m_packages.count(), 5);//We expect 5 Packages
+    QCOMPARE(m_packets.count(), 5);//We expect 5 Packets
     for(int x = 0;x < 5; ++x) {
-        QCOMPARE(m_packages[x], dataToSend[x]);
+        QCOMPARE(m_packets[x], dataToSend[x]);
     }
 
     delete m_reader;
@@ -163,14 +187,16 @@ void TestSslSocketLineReader::testTrustedDevice()
 void TestSslSocketLineReader::testUntrustedDevice()
 {
     int maxAttemps = 5;
-    QCOMPARE(true, m_server->hasPendingConnections());
     while(!m_server->hasPendingConnections() && maxAttemps > 0) {
         --maxAttemps;
         QTest::qSleep(1000);
     }
+    QCOMPARE(true, m_server->hasPendingConnections());
 
     QSslSocket* serverSocket = m_server->nextPendingConnection();
 
+    QSignalSpy serverEncryptedSpy(serverSocket, SIGNAL(encrypted()));
+
     QVERIFY2(serverSocket != 0, "Null socket returned by server");
     QVERIFY2(serverSocket->isOpen(), "Server socket already closed");
 
@@ -183,10 +209,27 @@ void TestSslSocketLineReader::testUntrustedDevice()
     m_clientSocket->setPeerVerifyName(QStringLiteral("Test Server"));
     m_clientSocket->setPeerVerifyMode(QSslSocket::QueryPeer);
 
-    connect(m_clientSocket, &QSslSocket::encrypted, &m_loop, &QEventLoop::quit);
+    int connected_sockets = 0;
+    auto connected_lambda = [&](){
+        connected_sockets++;
+        if (connected_sockets >= 2) {
+            m_loop.quit();
+        }
+    };
+    connect(serverSocket, &QSslSocket::encrypted, connected_lambda);
+    connect(m_clientSocket, &QSslSocket::encrypted, connected_lambda);
     serverSocket->startServerEncryption();
     m_clientSocket->startClientEncryption();
-    m_loop.exec();
+    m_loop.exec(); //Block until QEventLoop::quit gets called by the lambda
+
+    if (serverEncryptedSpy.count() < 1) {
+        for(int x = 0;x < 50; ++x) {
+            QVERIFY(serverEncryptedSpy.wait(100));
+            if (serverEncryptedSpy.count() > 0) {
+                break;
+            }
+        }
+    }
 
     QVERIFY2(m_clientSocket->isOpen(), "Client socket already closed");
     QVERIFY2(serverSocket->isOpen(), "Server socket already closed");
@@ -197,23 +240,23 @@ void TestSslSocketLineReader::testUntrustedDevice()
 
     QList<QByteArray> dataToSend;
     dataToSend << "foobar\n" << "barfoo\n" << "foobar?\n" << "\n" << "barfoo!\n" << "panda\n";
-    for (const QByteArray& line : dataToSend) {
+    for (const QByteArray& line : qAsConst(dataToSend)) {
             m_clientSocket->write(line);
         }
     m_clientSocket->flush();
 
-    m_packages.clear();
+    m_packets.clear();
 
     m_reader = new SocketLineReader(serverSocket, this);
-    connect(m_reader, &SocketLineReader::readyRead, this, &TestSslSocketLineReader::newPackage);
+    connect(m_reader, &SocketLineReader::readyRead, this, &TestSslSocketLineReader::newPacket);
     m_loop.exec();
 
     /* remove the empty line before compare */
     dataToSend.removeOne("\n");
 
-    QCOMPARE(m_packages.count(), 5);//We expect 5 Packages
+    QCOMPARE(m_packets.count(), 5);//We expect 5 Packets
     for(int x = 0;x < 5; ++x) {
-        QCOMPARE(m_packages[x], dataToSend[x]);
+        QCOMPARE(m_packets[x], dataToSend[x]);
     }
 
     delete m_reader;
@@ -226,6 +269,7 @@ void TestSslSocketLineReader::testTrustedDeviceWithWrongCertificate()
         --maxAttemps;
         QTest::qSleep(1000);
     }
+    QCOMPARE(true, m_server->hasPendingConnections());
 
     QSslSocket* serverSocket = m_server->nextPendingConnection();
 
@@ -261,7 +305,7 @@ void TestSslSocketLineReader::testTrustedDeviceWithWrongCertificate()
 
 }
 
-void TestSslSocketLineReader::newPackage()
+void TestSslSocketLineReader::newPacket()
 {
     if (!m_reader->bytesAvailable()) {
         return;
@@ -270,17 +314,23 @@ void TestSslSocketLineReader::newPackage()
     int maxLoops = 5;
     while(m_reader->bytesAvailable() > 0 && maxLoops > 0) {
         --maxLoops;
-        const QByteArray package = m_reader->readLine();
-        if (!package.isEmpty()) {
-            m_packages.append(package);
+        const QByteArray packet = m_reader->readLine();
+        if (!packet.isEmpty()) {
+            m_packets.append(packet);
         }
 
-        if (m_packages.count() == 5) {
+        if (m_packets.count() == 5) {
             m_loop.exit();
         }
     }
 }
 
+void TestSslSocketLineReader::testTimeout()
+{
+    m_loop.exit(-1);
+    QFAIL("Test Timed Out");
+}
+
 void TestSslSocketLineReader::setSocketAttributes(QSslSocket* socket, QString deviceName) {
 
     QDateTime startTime = QDateTime::currentDateTime();
